#include <NintendoSwitchControlLibrary.h>
#include "bmpdata.h"

//現在の座標の保持変数
int x_pos = 0;
int y_pos = 0;

//終了したかどうかのフラグ
bool finFlag = false;

//Splabannerからの引継ぎ変数、SetUpでロードする
uint16_t PRESS_WAIT_TIME   = 30;
uint16_t RELEASE_WAIT_TIME = 30;
uint8_t A_PUSH_COUNT = 10;
uint8_t OVER_PUSH_COUNT = 10;
bool FINISH_SAVE = true;


//bmpデータから指定座標のドット情報の取得 true:白 false:黒
bool GetDot(int x, int y){
  bool dot;
  uint8_t v = pgm_read_byte(&(Data[y][x/8])); //PROGMEMからのリード

  dot = false;
  if(x % 8 == 0) dot = (v & 0x80) > 0;
  if(x % 8 == 1) dot = (v & 0x40) > 0;
  if(x % 8 == 2) dot = (v & 0x20) > 0;
  if(x % 8 == 3) dot = (v & 0x10) > 0;
  if(x % 8 == 4) dot = (v & 0x08) > 0;
  if(x % 8 == 5) dot = (v & 0x04) > 0;
  if(x % 8 == 6) dot = (v & 0x02) > 0;
  if(x % 8 == 7) dot = (v & 0x01) > 0;

  return dot;
}

//ドット打ち
void DotBlack(){
  SwitchControlLibrary().pressButton(Button::A);
  SwitchControlLibrary().sendReport();   
  delay(PRESS_WAIT_TIME);
  SwitchControlLibrary().releaseButton(Button::A);
  SwitchControlLibrary().sendReport();   
  delay(RELEASE_WAIT_TIME);
}

//十字キーの入力
void pushHatEx(uint8_t hat){
  SwitchControlLibrary().pressHatButton(hat);
  SwitchControlLibrary().sendReport();   
  delay(PRESS_WAIT_TIME);
  SwitchControlLibrary().releaseHatButton();   
  SwitchControlLibrary().sendReport();   
  delay(RELEASE_WAIT_TIME);
}

void setup(){
   //Splabanner引継ぎ変数のロード
   //ウエイトタイムの初期値をロード
   PRESS_WAIT_TIME = pgm_read_word(&(Data[120][0]));
   RELEASE_WAIT_TIME = pgm_read_word(&(Data[120][2]));
   //Aボタンプッシュ回数
   A_PUSH_COUNT = pgm_read_byte(&(Data[120][4]));
   //ズレ防止用プッシュ回数
   OVER_PUSH_COUNT = pgm_read_byte(&(Data[120][5]));
   //書き込んだ後保存するか
   uint8_t f = pgm_read_byte(&(Data[120][6]));
   FINISH_SAVE = f > 0;
  
   //コントローラーの決定でAボタンを5回0.5秒ごと押す
   pushButton(Button::A, 500, A_PUSH_COUNT);
   
   //小さいペンに変える
   pushButton(Button::L, 500, 5);

   //ペン先を左上に持ってくる
   tiltLeftStick(Stick::MIN, Stick::MIN, 5000);

   //全面クリア
   pushButton(Button::LCLICK);
}

void loop(){

    if(y_pos >= 120) {
      //打ち終わったら保存処理する場合
      if(!finFlag && FINISH_SAVE){
        //保存するのに念のためマイナスを0.5秒毎10回押す
        pushButton(Button::MINUS, 500, 10);
        finFlag = true;
      }
      
      return; //y_posが119行を超えたら終わり
    }

    bool dot = GetDot(x_pos,y_pos);

    //黒の時ドットを打つ
    if( !dot ) DotBlack();
 
    //yが偶数の時は右へ　奇数は左へ
    if( y_pos % 2 == 0 ){
      //偶数
      if(x_pos >= 319 ){
        //右端到達改行
        for(int i=0;i< OVER_PUSH_COUNT ;i++) pushHatEx(Hat::RIGHT); //ズレ回避
        y_pos++;
        pushHatEx(Hat::DOWN);
      }else{
        //右へ進める
        x_pos++;
        pushHatEx(Hat::RIGHT);
      }
    }else{
      //奇数
      if(x_pos <= 0){
        //左端到達改行
        for(int i=0; i < OVER_PUSH_COUNT ;i++) pushHatEx(Hat::LEFT); //ズレ回避
        y_pos++;
        pushHatEx(Hat::DOWN);
      }else{
        //左へ進める
        x_pos--;
        pushHatEx(Hat::LEFT);
      }
    }
}
